@@ -6,18 +6,18 @@ usepackage_latex <- function(name, options = NULL, extra_lines = NULL) {
 
 sanitize_j <- function(j, x) {
   # regex
-  if (is.character(j) && length(j) == 1 && !is.null(meta(x, "colnames"))) {
-    j <- grep(j, meta(x, "colnames"), perl = TRUE)
+  if (is.character(j) && length(j) == 1 && !is.null(colnames(x))) {
+    j <- grep(j, colnames(x), perl = TRUE)
   # full names
-  } else if (is.character(j) && length(j) > 1 && !is.null(meta(x, "colnames"))) {
-    bad <- setdiff(j, meta(x, "colnames"))
+  } else if (is.character(j) && length(j) > 1 && !is.null(colnames(x))) {
+    bad <- setdiff(j, colnames(x))
     if (length(bad) > 0) {
       msg <- sprintf("Missing columns: %s", paste(bad, collapse = ", "))
       stop(msg, call. = FALSE)
     }
-    j <- which(meta(x, "colnames") %in% j)
+    j <- which(colnames(x) %in% j)
   } else {
-    assert_integerish(j, lower = 1, upper = meta(x, "ncols"), null.ok = TRUE)
+    assert_integerish(j, lower = 1, upper = ncol(x), null.ok = TRUE)
   }
   return(j)
 }
@@ -43,6 +43,7 @@ sanitize_output <- function(output) {
         "\\usepackage[normalem]{ulem}",
         "\\usepackage{graphicx}",
         "\\UseTblrLibrary{booktabs}",
+        "\\UseTblrLibrary{siunitx}",
         "\\NewTableCommand{\\tinytableDefineColor}[3]{\\definecolor{#1}{#2}{#3}}",
         "\\newcommand{\\tinytableTabularrayUnderline}[1]{\\underline{#1}}",
         "\\newcommand{\\tinytableTabularrayStrikeout}[1]{\\sout{#1}}"
@@ -50,7 +51,7 @@ sanitize_output <- function(output) {
       )
       if (is.null(output)) out <- "latex"
 
-    } else if (isTRUE(knitr::pandoc_to() == "html")) {
+    } else if (isTRUE(knitr::pandoc_to() %in% c("html", "revealjs"))) {
       if (is.null(output)) out <- "html"
 
     } else if (isTRUE(knitr::pandoc_to() == "typst")) {
@@ -184,9 +185,13 @@ assert_numeric <- function(x, len = NULL, lower = NULL, upper = NULL, null.ok =
 }
 
 
-assert_data_frame <- function(x, name = as.character(substitute(x))) {
+assert_data_frame <- function(x, min_rows = 0, min_cols = 0, name = as.character(substitute(x))) {
   msg <- sprintf("`%s` must be a data.frame.", name)
   if (!is.data.frame(x)) stop(msg, call. = FALSE)
+  msg <- sprintf("Number of rows in `%s` must be at least `%s`", name, min_rows)
+  if (nrow(x) < min_rows) stop(msg, call. = FALSE)
+  msg <- sprintf("Number of columns in `%s` must be at least `%s`", name, min_cols)
+  if (ncol(x) < min_cols) stop(msg, call. = FALSE)
 }
 
 
@@ -234,6 +239,8 @@ check_atomic_vector<- function(x, null.ok = FALSE, name = as.character(substitut
   flag <- is.atomic(x) && is.vector(x) && !is.list(x)
   if (flag) {
     out <- TRUE
+  } else if (is.factor(x) && is.null(dim(x))) {
+    out <- TRUE
   } else {
     out <- sprintf("`%s` must be an atomic vector.", name)
   }
@@ -249,7 +256,7 @@ assert_class <- function(x, classname) {
 }
 
 
-sanity_notes <- function(notes) {
+sanitize_notes <- function(notes) {
   if (is.character(notes) && length(notes) > 0) {
     notes <- as.list(notes)
   }
@@ -271,4 +278,5 @@ sanity_notes <- function(notes) {
       stop("`notes` includes invalid elements. Please refer to the documentation for details.", call. = FALSE)
     }
   }
+  return(notes)
 }
